# -*- coding: utf-8 -*-
import os
import time
import oss2
import json

from resource import parse_pji_image

import logging

path1 = '/mnt/disk001/dcl_data_process/src/python2/pjibot/'

logging.basicConfig(filename=path1 + 'log/camera-pjibot_guide.log', level=logging.INFO,
                    format='%(asctime)s - %(levelname)s - %(message)s')

key1 = 'pjibot/'
sleep_time = 2  # 每多少秒扫描一次

error_bag_json = "/mnt/disk001/dcl_data_process/src/python2/pjibot/camera-errorBag.json"


def parse_json_to_string_array(file_path):
    try:
        # 打开并读取JSON文件（Python 2中不支持encoding参数，需要使用codecs模块或处理文件读取后的编码）
        with open(file_path, 'r') as file:
            # 读取文件内容
            file_content = file.read()
            # 解析JSON内容（Python 2中json.loads用于解析字符串）
            data = json.loads(file_content.decode('utf-8'))  # 假设文件是UTF-8编码，这里需要手动解码

        # 检查数据是否是一个列表，并且列表中的元素是否是字符串
        if isinstance(data, list):
            for item in data:
                if not isinstance(item, basestring):  # Python 2中字符串类型包括str和unicode，用basestring检查
                    raise ValueError("JSON数组中的元素不是字符串")
            return data
        else:
            return []
    except Exception as e:
        return []


def list_to_json_file(data, file_path):
    """
    将列表转换为JSON格式并写入指定的文件路径。
    如果文件已存在，则覆盖它。

    参数:
    data (list): 要转换为JSON的列表。
    file_path (str): 要写入JSON数据的文件路径。
    """
    # 将列表转换为JSON格式的字符串，并确保输出为UTF-8编码的字符串
    json_data = json.dumps(data, ensure_ascii=False, indent=4)
    json_data_utf8 = json_data.encode('utf-8')  # 编码为UTF-8

    # 以写入模式打开文件，如果文件已存在则覆盖
    with open(file_path, 'w') as file:
        # 将UTF-8编码的JSON字符串写入文件
        file.write(json_data_utf8)


def parse_to_mp4(merged_bag_file_path, parse_prefix, local_parse_dir, local_delete_list):
    try:
        local_mp4_dir1 = parse_pji_image.parse1('/ob_camera_01/color/image_raw', merged_bag_file_path,
                                                local_parse_dir + '/camera1/')
        mp4_file_name1 = 'camera'
        local_mp4_file_path1 = local_mp4_dir1 + '/' + mp4_file_name1 + '.mp4'
        local_delete_list.append(local_mp4_file_path1)
        oss_csv_object_key1 = parse_prefix + mp4_file_name1 + '.mp4'
        bucket.put_object_from_file(oss_csv_object_key1, local_mp4_file_path1)

        local_mp4_dir2 = parse_pji_image.parse2('/ob_camera_02/color/image_raw', merged_bag_file_path,
                                                local_parse_dir + '/camera2/')
        mp4_file_name2 = 'camera2'
        local_mp4_file_path2 = local_mp4_dir2 + '/' + mp4_file_name2 + '.mp4'
        local_delete_list.append(local_mp4_file_path2)
        oss_csv_object_key2 = parse_prefix + mp4_file_name2 + '.mp4'
        bucket.put_object_from_file(oss_csv_object_key2, local_mp4_file_path2)

    except Exception as e2:
        error_bag_list = parse_json_to_string_array(error_bag_json)
        error_bag_list.append(parse_prefix)
        list_to_json_file(error_bag_list, error_bag_json)
        # 当出现异常时执行的代码
        logging.exception("生成摄像头视频报错: %s", e2)


# ------- 获取合并之后的bag包，解析出csv -------
if __name__ == '__main__':
    # 1 创建阿里云对象
    auth = oss2.Auth('n8glvFGS25MrLY7j', 'xZ2Fozoarpfw0z28FUhtg8cu0yDc5d')
    endpoint = 'oss-cn-beijing-gqzl-d01-a.ops.gqzl-cloud.com'
    bucket = oss2.Bucket(auth, endpoint, 'pji-bucket1')

    while True:
        logging.info("开始新一轮扫描")
        try:
            local_delete_list = []
            oss_delete_list = []
            prefix_list = []
            # 2 获取已经上传完成的所有目录并分组
            for obj1 in oss2.ObjectIterator(bucket, prefix=key1):
                # 获取合并后的包
                merged_bag_object_key = str(obj1.key)
                # print(f'判断1{merged_bag_object_key}')
                if 'data_merge' in str(obj1.key) and str(obj1.key).endswith('.bag'):
                    merged_bag_object_key_split = merged_bag_object_key.split('/')
                    merged_prefix = '/'.join(merged_bag_object_key_split[:-1])
                    parse_prefix = merged_prefix.replace('data_merge', 'data_parse')
                    parse_prefix_full = merged_bag_object_key.replace('data_merge', 'data_parse')[:-4] + '/'
                    camera_done = False
                    camera2_done = False
                    callback_done = False
                    for obj2 in oss2.ObjectIterator(bucket, prefix=parse_prefix_full):
                        if '/callback.json' in str(obj2.key):
                            callback_done = True
                        if '/camera.mp4' in str(obj2.key):
                            camera_done = True
                        if '/camera2.mp4' in str(obj2.key):
                            camera2_done = True
                    if camera_done and camera2_done:
                        # logging.info("存在摄像头视频: %s" % str(parse_prefix_full))
                        continue
                    if not callback_done:  # 不存在存在callback.json的就不处理了
                        # logging.info("数据已回调完成，不需要处理: %s" % str(parse_prefix_full))
                        continue
                    error_bag_list = parse_json_to_string_array(error_bag_json)
                    if parse_prefix_full in error_bag_list:
                        continue
                    logging.info(str(merged_bag_object_key), "等待处理到：", str(parse_prefix_full))
                    local_merged_bag_path = path1 + 'camera/' + merged_bag_object_key
                    local_merged_dir = '/'.join(local_merged_bag_path.split('/')[:-1])
                    local_parse_dir = local_merged_dir.replace('data_merge', 'data_parse')
                    if not os.path.exists(local_merged_dir):
                        os.makedirs(local_merged_dir)
                    if not os.path.exists(local_parse_dir):
                        os.makedirs(local_parse_dir)
                    merged_bag_full_name = merged_bag_object_key_split[-1]
                    merged_bag_name = merged_bag_full_name.split('.')[0]
                    try:
                        bucket.get_object_to_file(merged_bag_object_key, local_merged_bag_path)
                    except Exception as e:
                        logging.exception("下载合并后的bag包失败: %s" % str(e))

                    local_delete_list.append(local_merged_bag_path)
                    # 2 生成 pos_orig.csv 和 pos_hmi.csv
                    parse_to_mp4(local_merged_bag_path, parse_prefix_full, local_parse_dir, local_delete_list)

            # 删除本地临时文件
            if len(local_delete_list) > 0:
                for local_delete in local_delete_list:
                    try:
                        os.remove(local_delete)
                    except Exception as e:
                        pass
                        # logging.exception("捕获到一个异常: %s" % str(e))
            time.sleep(sleep_time)
        except Exception as e:
            logging.exception("全局错误处理: %s" % str(e))
